#version 330 core

//Interpolierte Werte vom Vertexshader uebergeben
in vec2 fUVs;
in vec3 position_wspc;
in vec3 normal_cspc;
in vec3 viewdir_cspc;
//in vec3 lightdir_cspc;

//Output (Farbe des Pixels)
out vec4 outcolor;

struct lighting{
	vec3 position;
	vec3 color;
	float power;
};

layout (std140) uniform Material {
	vec4 diffuse;
	vec4 ambient;
	vec4 specular;
	vec4 emissive;
	float shininess;
	int texCount;
};

//Konstante Werte frs ganze Mesh
uniform sampler2D colorTexture;
//uniform vec3 lPosition_wspc;
uniform lighting light[3];
uniform mat4 viewMatrix;
uniform int numberoflights;

void main(){

	vec3 mDiffuseColor;
	vec3 mAmbientColor;
	vec3 mSpecularColor;



	//Materialwerte 
	//if(texCount == 0){
	//	mDiffuseColor= diffuse.rgb;
	//	mAmbientColor= ambient.rgb;
	//	mSpecularColor= specular.rgb;
	//} else {
		mDiffuseColor= texture(colorTexture, fUVs).rgb;
		mAmbientColor= vec3(0.1, 0.1, 0.1) * mDiffuseColor;
		mSpecularColor= vec3(0.3, 0.3, 0.3);
	//}

	vec3 color = vec3(0.0, 0.0, 0.0);
	
	for(int i=0; i<numberoflights; i++){
		//Lichtwerte (hardgecoded, spaeter ueber in bestimmbar)
	
		//Lichtwerte werden ausgelesen
		vec3 lColor= light[i].color;
		float lPower= light[i].power;

		vec3 lPosition_cspc= vec3(viewMatrix * vec4(light[i].position,1.0));
		vec3 lightdir_cspc= lPosition_cspc + viewdir_cspc;


		//Distanz zum Licht
		float distance= length(lPosition_cspc - position_wspc);
		//vec3 lightdir_cspc= normalize(distance);

		//Normale des zu berechnenden Fragments(N), im Cameraspace
		vec3 n= normalize(normal_cspc);

		//Richtung des Lichtes(L)
		vec3 l= normalize(lightdir_cspc);

		//Cosinus des Winkels zwischen der Normale und der Lichtrichtung(cosNL)
		//Falls der Winkel negativ ist wird er auf 0,1 gesetzt 
		float cosNL= max( dot(l,n), 0.0);

		//Viewvektor 
		//vec3 V= normalize(viewdir_cspc);
		//Halbvektor H
		//Halbvektor
		//vec3 H= normalize(V + l);		
		//float cosNH= max( dot(n,H), 0.0);

		vec3 E= normalize(viewdir_cspc);
		vec3 R= reflect(-l,n);
		float cosAlpha= max(dot(E,R), 0.0);

		color+=
		//Ambient: simuliert indirekte Beleuchtung
		//mAmbientColor +
		//Diffuse: "Farbe" des Objektes
		mDiffuseColor * lColor * lPower * cosNL/(distance*distance) +
		//Specular: reflektierter Glanzpunkt
		mSpecularColor * lColor * lPower * pow(cosAlpha,5)/(distance*distance);

		
	}

	//color+= mAmbientColor;
	outcolor=vec4(color, 1.0);
}